Method: Smooth::Query#execute
- Defined in:
- lib/smooth/query.rb
#execute ⇒ Object
To customize the filtering behavior of the query you can supply your own execute method body. The execute method is expected to mutate the value of the self.scope or @scope instance variable and to return something which can be serialized as JSON using the Smooth::Serializer
11 12 13 14 |
# File 'lib/smooth/query.rb', line 11 def execute apply_filters scope end |